%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%                                   genRosenblatt4
%
%  function generating a time series from a Rosenblatt process a limit
%  of sum of a Fractional Gaussian Noise (generated from the circulant matrix procedure)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Input	      n    : length of the time series (positive integer)
%    		      H    : self similarity parameter (1/2<H<1) 
%
%  Output	  	traj   : a sample of length n of a Rosenblatt which is H-self similar process 
%               increm : increments of the Rosenblatt path  
%
%  Example     traj=genRosenblatt4(1000,0.7)
%
%		Created by Bardet, 2008, improved in 2012
%                          
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [traj,increm]=genRosenblatt4(n,H)

m=200;
[fGn,fBm]=genFARIMAEM4(n*m,H/2,0,0);
%plot(fGn)
% plot(fBm)
for t=1:n
traj(t)=(n*m)^(-H)*sum(fGn(1:(t*m)).^2-1);
end
increm=traj(2:n)-traj(1:(n-1));


function [traj,fBm]=genFARIMAEM4(n,d,phi,psi)

if (nargin < 3)
   phi=[]; 
   psi=[];
end;
m=2^nextpow2(2*(n-1));
c=farimacvEM(d,phi,psi,1,m,1);
c=[c(1:(end-1))' fliplr(c(2:end)')]; 
vp=fft(c); 
%=================================================
%		Generation des trajectoires
%=================================================
m  		=  length(vp);
ar		=  randn(1,m/2 + 1);  	
ai		=  randn(1,m/2 + 1);			
ar(1) 		=  sqrt(2) * ar(1);
ar(m/2 + 1) 	=  sqrt(2) * ar(m/2 + 1);
ai(1) 		=  0;
ai(m/2 + 1) 	=  0;
ar		= [ar(1:m/2 + 1)  ar(m/2:-1:2)];
aic 		= - ai;
ai		= [ai(1:m/2 + 1)  aic(m/2:-1:2)];


a		= ar+i*ai;
a 		= sqrt(vp) .* a;
a 		= real(fft(a));
a 		= (1/(sqrt(2 * m))) * a;

traj 	= a(1:n);
fBm=cumsum(traj);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%FARIMACV  Compute the auto-covariance function of a Fractional ARIMA model
% using the of Sowell (1992), Maximum Likelihood estimation of stationary
% univariate stationary time-series, J. Econometrics, vol. 53, pp. 165-188.
% Model:  y_t + \theta(1) y_{t-1} + \cdots + \theta(p) y_{t-p} 
%   = \epsilon_t + \phi(1) * epsilon_{t-1} + \cdots + \phi(q) \epsilon_{t-q}.
% Syntax: [covcoeff,psd] = farimacv(d,phi,theta,var,nobs,check);
% Input -
%   phi   : coefficients of the AR part 
%   theta : coefficients of the MA part
%   var   : innovation variance
%   d     : fractional exponent
%   nobs  : number of output covariance samples
%   check : set to 1 to check the results
% Output -
%   covcoeff : covariance coefficient of the fractional ARMA model
%   psd      : spectral density - psd(:,1): estimated from the discrete Fourier
%              transform of the autocovariance sequence - psd(:,2): estimated
%              from the parameters of the FARIMA model (if check == 1 only).
% Warning: use theta and [1 phi] for compatibility with MATLAB functions
%   like filter().

% Author: E. Moulines, moulines@tsi.enst.fr, 10-Dec-1997 / 08-Feb-1999
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [c,psd] = farimacvEM(d,phi,theta,var,nobs,check);

% Input args.
error(nargchk(5, 6, nargin));
if (nargin < 6)
  check = 0;
end;

% Order of the AR/MA part
phi= phi(:);
theta= theta(:);
p= length(phi);
q= length(theta);
phitmp= [1; phi]; 
thetatmp= [1; theta];

% Compute psi(k) = sum_{s=|k|}^q \theta_s theta_{s-|k|}, psi(-k)=psi(k)
psi= conv(thetatmp,flipud(thetatmp));

% Autocovariance coefficients of the AutoRegressive model
if (p ~= 0)
  % First calculate the roots of the polynomial phi(1)*z^p + \cdots + phi(p+1)
  rho=roots(phitmp);   

  % Check to see that none of these roots are too close to the unit circle
  if (max(abs(rho)) > 0.99)
    disp('fracsigma: root too close to the unit circle');
    return   
  end

  % Computation of the $\xi_i$ coefficients:
  xi =zeros(p,1);
  for ii=1:p,
    xi(ii)= 1/(prod(1 - rho(1:ii-1)/rho(ii))*prod(1-rho(ii+1:p)/rho(ii))*prod(1-rho*rho(ii))); 
  end
end

% Case 1: non fractional models (too low values for $d$ causes instability)
T= q+nobs+1;
coef= 1;
if (abs(d) < 1e-2)
   c= zeros(1,T);
   if (p ~= 0)
      for ii=1:p,
         c= c + xi(ii)*rho(ii).^(0:(T-1));
      end
      c= real(c);
   else
      c(1)= 1;
   end;
   c= c(:);
else
   %Case 2: fractional models 
   c= zeros(T,1);
   j=0;
   c(1)=1;
   for m=2:T+1,
      c(m) = c(m-1) * (d+j) / (1-d+j);
      j= j+1;
   end

   % Store cinv in reverse order - speed up computation 
   cinv= flipud(c(2:T+1));
   % Scale factor for variance
   coef= var*exp(gammaln(1-2*d)-2*gammaln(1-d));
   if (p ~= 0)
      % Computation of S(t;rho)
      Stot= zeros(T,1);
      for ii=1:p,
         U = zeros(T,1);
         %U0= hygernd(d,(1-d),rho(ii),10^5);
         U0= hypergm(d,(1-d),rho(ii),10^5);
         U(1)= U0;
         U= filter(1, [1 -rho(ii)],[0;c(2:T)],U0);
         %V0= cinv(1)*hygernd(d+T+1,2-d+T,rho(ii),10^5);
         V0= cinv(1)*hypergm(d+T+1,2-d+T,rho(ii),10^5);
         V= filter(1, [1 -rho(ii)],cinv,V0);
         Stot= Stot + xi(ii)*(U(1:T)+rho(ii)*flipud(V(1:T)));
      end  
      c= Stot;
  end
end
c= [flipud(c(2:q+1)); c];

% Convolve psi and c
c= conv(psi,c);
c= coef*c(2*q+1:2*q+nobs);

% Check the results
psd= zeros(nobs-1,2);
if (check == 1)
   ctmp= [flipud(c(2:nobs)); c(1:nobs)];
   ctmp= abs(fft(ctmp,2*nobs-1))/2/pi;
   psd(:,1)= ctmp(2:nobs);
   psd(:,2)= farimaspEM(d,phi,theta,2*pi*(1:(nobs-1))'/(2*nobs-1));
end
c= real(c);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%FARIMASP  Evaluate the spectral density function of a FARIMA process at
% given frequency  points
% Syntax: dspfarima = farimasp(d,phi,theta,fs);
% Input:
%   d       : fractional difference coefficient
%   phi     : AR coefficients (same convention than in fracsigma.m)
%   theta   : MA coefficients
%   innovar : innovation variance
% Output:
%   dspfarima : spectral density function of the farima process
% Remark: Features 1/(2*pi) noramlization of the psd.

% Author: E. Moulines, moulines@tsi.enst.fr, 10-Dec-1997 / 08-Feb-1999
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function dspfarima = farimaspEM(d,phi,theta,fs);

% Input args.
error(nargchk(4, 4, nargin));

phi= phi(:);
theta= theta(:);
phi= [1; phi];
theta= [1; theta];
% Evaluation of the spectral density (on the frequency grid fs)
i = sqrt(-1);
efs= exp(i*fs);
dspfarima = (2*sin(fs/2)).^(-2*d).*abs(polyval(theta,efs)).^2./ ...
	    abs(polyval(phi,efs)).^2/2/pi;

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%����
    
    function hyp= hypergm(a,b,z,niter,feps)
% PURPOSE: The hypergeometric function
% -----------------------------------------------------
% USAGE:     hyp= hypergm(a,b,z,niter,feps)
% where:     a, b: the variables of function:a and c, b =1
%            z: the roots of the AR polynomial
%            niter   : the number of iterations
%            feps : the required precision
% -----------------------------------------------------
% RETURNS:   result= The likelihoods of Time series
% -----------------------------------------------------             
% NOTE:      The function is used in the maximum likelihood procedure.
% -----------------------------------------------------
% ACKNOWLEDGEMENT: The original ARFIMA code was written by Jurgen A Doornik
% -----------------------------------------------------

% The matlab port written and changed by
% Ying Ma
% yingma@email.com

error(nargchk(4, 5, nargin));
if (nargin <= 4) 
   feps = eps;
end;

% Initialization
hyp= 1;
inc= 1;
for ii=0:niter,
    inc= inc*z*(a+ii)/(b+ii);
    hyp= hyp+inc;
    if (abs(inc) < feps) 
       return; 
    end;
end
